local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikUnitFrame = BaikCore.BaikUnitFrame
local Assert = BaikCore.Assert
local Database = BaikCore.Database
local Hook = BaikCore.Hook
local Log = BaikCore.Log
local Option = BaikCore.Option
local Reload = BaikCore.Reload

-- Add Module
local Portrait = BaikUnitFrame:NewModule("Portrait", "AceHook-3.0")

-- Constants
local CLASS_TEXTURE = "Interface\\TargetingFrame\\UI-Classes-Circles"
local DEFAULT_TEXCOORD = {0.0, 1.0, 0.0, 1.0}

local CLASS_ICON_DB = "portrait_class_icon"
local CLASS_DEFAULT = false

local SPELL_ICON_DB = "spellcast_icon"
local SPELL_DEFAULT = true

-- Private Methods
local UNIT_FRAME = {
    [CastingBarFrame] = PlayerFrame,
    [TargetFrameSpellBar] = TargetFrame,
    [FocusFrameSpellBar] = FocusFrame,
}

local function Reset(unitFrame)
    UnitFramePortrait_Update(unitFrame)
end

local function SetClassIcon(unitFrame)
    -- Ensures unit is a player
    local unit = unitFrame.unit
    local portrait = unitFrame.portrait
    if not portrait then
        return
    end

    if UnitIsPlayer(unit) then
        -- Attempts to get player's class textures
        local texCoord = CLASS_ICON_TCOORDS[select(2, UnitClass(unit))]
        if texCoord then
            -- Updates portrait
            portrait:SetTexture(CLASS_TEXTURE)
            portrait:SetTexCoord(unpack(texCoord))
        end
    elseif unit and portrait then
        SetPortraitTexture(portrait, unit)
        portrait:SetTexCoord(unpack(DEFAULT_TEXCOORD))
    end
end

local function SetIcon(unitFrame, texture)
    local portrait = unitFrame.portrait
    if not portrait then
        return
    end
    portrait:SetTexCoord(unpack(DEFAULT_TEXCOORD))
    SetPortraitToTexture(portrait, texture)
end

local function PostHook_SetPortraitToTexture(frame, ...)
    SetClassIcon(frame)
end

local function PreHook_CastingBarFrame_SetTexture(icon, ...)
    local castbar = icon:GetParent()
    local unitFrame = UNIT_FRAME[castbar]
    if unitFrame then
        SetIcon(unitFrame, icon:GetTexture())
    end
end

local function PreHook_CastingBarFrame_OnHide(frame, ...)
    local unitFrame = UNIT_FRAME[frame]
    if unitFrame then
        Reset(unitFrame)
    end
end

local function HookClassIcon(value)
    if value then
        Hook:EnsureSecureHook(Portrait, "UnitFramePortrait_Update",
                              PostHook_SetPortraitToTexture)
    else
        Hook:EnsureUnhook(Portrait, "UnitFramePortrait_Update",
                          PostHook_SetPortraitToTexture)
    end
end

local function HookSpellIcon(value)
    for castbar, _ in pairs(UNIT_FRAME) do
        if value then
            Hook:EnsureSecureHook(Portrait, castbar.Icon, "SetTexture",
                                  PreHook_CastingBarFrame_SetTexture)
            Hook:EnsureHookScript(Portrait, castbar, "OnHide",
                                  PreHook_CastingBarFrame_OnHide)
        else
            Hook:EnsureUnhook(Portrait, castbar.Icon, "SetTexture")
            Hook:EnsureUnhook(Portrait, castbar, "OnHide")
        end
    end
end

local function GetOpt()
    return {
        name ="Portrait",
        type = "group",
        childGroups = "tree",
        args = {
            class_icon = {
                name = "Class Icons",
                type = "toggle",
                get = function(...)
                    return Database:Get(CLASS_ICON_DB)
                end,
                set = function(info, value, ...)
                    Database:Set(CLASS_ICON_DB, value)
                    HookClassIcon(value)
                    Reload:Enable()
                end
            },
            spell_icon = {
                name = "Spell Icons",
                type = "toggle",
                get = function(...)
                    return Database:Get(SPELL_ICON_DB)
                end,
                set = function(info, value, ...)
                    Database:Set(SPELL_ICON_DB, value)
                    HookSpellIcon(value)
                end
            }
        }
    }
end

-- Ace3 Callbacks
function Portrait:OnInitialize()
    Log:v("Portrait_OnInitialize")
    Database:InitIfEmpty(CLASS_ICON_DB, CLASS_DEFAULT)
    Database:InitIfEmpty(SPELL_ICON_DB, SPELL_DEFAULT)
    Option:AddOption(BaikUnitFrame.OPTION_NAME, "portrait", GetOpt())
end

function Portrait:OnEnable()
    Log:v("Portrait_OnEnable")
    HookClassIcon(Database:Get(CLASS_ICON_DB))
    HookSpellIcon(Database:Get(SPELL_ICON_DB))
end

function Portrait:OnDisable()
    Log:v("Portrait_OnDisable")
    self:UnhookAll()
end
